from cmath import nan
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
import seaborn as sns
import sys
import pathlib
import glob
from tkinter import filedialog
import tkinter
import multiprocessing as mp


class Graph:
    def loadData(self, filename=None):
        if filename == None:
            tkinter.Tk().withdraw()
            SelectedFile = filedialog.askopenfilename(
                title="Open graph.csv file",
                filetypes=[('csvファイル', '*graph.csv')],
                initialdir=pathlib.Path(sys.argv[0]).parent)
        else:
            SelectedFile = filename

        if ('graph.csv' in str(SelectedFile)):
            self.graph_fullpath = pathlib.Path(SelectedFile)
        else:
            print("No file selected.")
            exit()

        info_type = {
            'uid': 'str',
            'date': 'str',
            'start_time': 'str',
            'measured_eye': 'str',
            'using_eye': 'str',
            'NumberOfStim': 'int',
            'CueSize[deg]': 'float',
            'EyeWindow[deg]': 'str',
            'RewardType': 'str',
            'time': 'float',
            'time_cpu': 'float',
            'dio': 'int',
            'horizontal': 'float',
            'vertical': 'float',
            'pupil': 'float',
            'counter': 'int',
            'eye open ratio': 'float',
            'densityx': 'float',
            'rfw': 'float',
            'rfh': 'float',
            'h_mean': 'float',
            'v_mean': 'float',
            'p_mean': 'float',
            'trial': 'int',
            'result': 'str',
            'StimNum': 'int',
            'Type': 'str,',
            'Event': 'str',
            'Display': 'str',
            'TS1on': 'float',
            'TS1off': 'float',
            'EKon': 'float', # Start time of Pre-stim
            'ES1on': 'float',# Stim-on
            'ES1off': 'float',
            'ERon': 'float',
            'EITIon': 'float',
        }
        self.graph = pd.read_csv(
            self.graph_fullpath, dtype=info_type, engine='python')
        self.graphNR = pd.read_csv(
            self.graph_fullpath, dtype=info_type, engine='python')
        rfw_median = self.graph['rfw'].median()
        rfh_median = self.graph['rfh'].median()

        self.graphNR = self.graph[(
            (rfw_median-5) < self.graph['rfw']) & (self.graph['rfw'] < (rfw_median+5))]
        self.graphNR = self.graphNR[(
            (rfh_median-5) < self.graphNR['rfh']) & (self.graphNR['rfh'] < (rfh_median+5))]
        self.graphNR = self.graphNR[self.graphNR['pupil'] != 0]
        self.graphNR['h_mean'] = self.graphNR['horizontal'].rolling(
            10).mean()
        self.graphNR['v_mean'] = self.graphNR['vertical'].rolling(10).mean()

        self.measured_eye = self.graph.loc[0, 'measured_eye']
        self.using_eye = self.graph.loc[0, 'using_eye']
        self.date = self.graph.loc[0, 'date']
        self.ti = self.graph.loc[0, 'start_time']
        self.rewardtype=self.graph.loc[0,'RewardType']
        self.using_eye=self.graph.loc[0,'using_eye']
        self.measured_eye=self.graph.loc[0,'measured_eye']

        self.types = list(set(self.graph['Type']))
        self.s_type = [s for s in self.types if 'S' in s]
        self.f_type = [f for f in self.types if 'F' in f]
        self.f_type.sort()
        self.types = self.s_type+self.f_type

        return True

    def mk_pdfdir(self):
        current_dir=current_dir = pathlib.Path(sys.argv[0]).parent
        LL=current_dir/'pdf'/'L'/'L'
        RR=current_dir/'pdf'/'R'/'R'
        for p in [LL,RR]:
            p.mkdir(parents=True,exist_ok=True)

    def __init__(self, filename=None):
        self.loadData(filename)
        self.mk_pdfdir()

    def del_ref_outofrange(self):
        rfw_median = self.graph['rfw'].median()
        rfh_median = self.graph['rfh'].median()
        self.graph = self.graph[(
            (rfw_median-5) < self.graph['rfw']) & (self.graph['rfw'] < (rfw_median+5))]
        self.graph = self.graph[(
            (rfh_median-5) < self.graph['rfh']) & (self.graph['rfh'] < (rfh_median+5))]

    def ref_histogram(self):
        rfw_median = self.graph['rfw'].median()
        rfh_median = self.graph['rfh'].median()

        ax = sns.histplot(data=self.graph, x="rfw", binwidth=1)
        ax.axvline(rfw_median)
        plt.show()
        ax = sns.histplot(data=self.graph, x="rfh", binwidth=1)
        ax.axvline(rfh_median)
        plt.show()

    def delblink(self):
        self.graph = self.graph[self.graph['eye open ratio'] > 0.6]

    def GetDuration(self, s, sign=1.0):
        duration = {
            'EyeWait': (0.5, 1),
            'EyeKeep': (0.4, 0.5),
            'Stim1on': (0.35, 0.5),
        }
        mi, ma = duration[s]
        return (mi*sign, ma*sign)

    def GetCommandTupple(self, type):
	# correct trials aligned at Stim-on
        S1 = ({'S': 'EyeKeep', 'E': 'Stim1on', 'Zero': 'ES1on'},
              )

	# error trials aligned at the beginnig of Pre-stim
        F0 = ({'S': 'EyeWait', 'E': 'EyeKeep', 'Zero': 'EKon'}, 
              )

	# error trials aligned at the Stim-on
        F1 = ({'S': 'EyeKeep', 'E': 'Stim1on', 'Zero': 'ES1on'},
              )

        com_dict = {
            'S1': S1, 
            'F0': F0, 
	    'F1': F1, 
        }
        return com_dict[type]

    def GetPdfFileName(self):
        current_dir = pathlib.Path(sys.argv[0]).parent
        storedir=current_dir/'pdf'/self.measured_eye/self.using_eye
        filename = str(self.date)+'_'+str(self.ti)+'.pdf'
        fullpathname=storedir/filename
        return fullpathname

    def GetA4size(self):
        return (11.69/2, 8.27*2/3)
        

    def GetTitle(self, and_conditions, loop, trialnum):
        tstr = self.date+'_'+self.ti+' '+self.measured_eye+'/'+self.using_eye+' ['+self.rewardtype+']\n'

        for key in and_conditions.keys():
            tstr = tstr+and_conditions[key]+' '
        tstr = tstr+'N:'+str(trialnum) + '\n'
        tstr = tstr+loop['S']+' -['+loop['Zero']+']- '+loop['E']
        return tstr

    def MakeGraph(self, df, dfNR, and_conditions, loop, trialnum):
        prmind, prd = self.GetDuration(loop['S'], -1)
        pomind, pod = self.GetDuration(loop['E'])
        fig, ax = plt.subplots(2, 1, figsize=self.GetA4size())

        manum = int((pod-prd)*10+1)

        sns.lineplot(x=loop['Zero'], y='h_mean', data=dfNR, ax=ax[0],
                     units='trial', estimator=None, lw=1, color=(0, 0, 0), alpha=0.2)
        sns.lineplot(x=loop['Zero'], y='v_mean', data=dfNR,  ax=ax[1],
                     units='trial', estimator=None, lw=1, color=(0, 0, 0), alpha=0.2)


        for j in [0,1]:
            for i in [0]:
                #ax[j][i].set_rasterized(True)
                ax[j].set_xticks(np.linspace(
                    start=prd, stop=pod, num=manum))
                ax[j].set_xticks(np.linspace(
                    start=prd, stop=pod, num=manum*2-1), minor=True)
                ax[j].grid(which="major", alpha=0.6)
                ax[j].grid(which="minor", alpha=0.3)
                ax[j].set_xlabel('Time [sec]') 
        for i in [0]:
            ax[0].set_ylabel('Horizontal [deg]') 
            ax[1].set_ylabel('Vertical [deg]') 
        ax[0].set_ylim([-10, 10]) 
        ax[1].set_ylim([-10, 10]) 
        ax[0].set_xlim([-0.2, 0.5]) 
        ax[1].set_xlim([-0.2, 0.5]) 
        fig.suptitle(self.GetTitle(and_conditions, loop, trialnum))
        fig.tight_layout()

    def MakeOneUnit(self, pdf_page, and_conditions):
        adf, adfNR = True, True
        for key in and_conditions:
            adf = adf & (self.graph[key] == and_conditions[key])
            adfNR = adfNR & (self.graphNR[key] == and_conditions[key])
        trialnum = self.graph[adf]['trial'].nunique()
        if trialnum == 0:
            return
        for loop in self.GetCommandTupple(and_conditions['Type']):
            df = self.graph[adf]
            dfNR = self.graphNR[adfNR]
            df = df[(df['Event'] == loop['S']) | (df['Event'] == loop['E'])]
            dfNR = dfNR[(dfNR['Event'] == loop['S']) |
                        (dfNR['Event'] == loop['E'])]
            self.MakeGraph(df, dfNR, and_conditions, loop, trialnum)
            pdf_page.savefig(dpi=300)
            plt.clf()


    def PngSave(self):
        if (len(self.s_type) == 0):
            print('No Success Data.')
            return
        pdf_page = PdfPages(self.GetPdfFileName())
        sns.set_theme(style="whitegrid")

        Rtype = self.s_type[0]
        self.MakeOneUnit(pdf_page, {'Type': Rtype, 'EyeWindow[deg]': '8x8'})
        self.MakeOneUnit(
            pdf_page, {'Type': Rtype, 'EyeWindow[deg]': 'EyeFree'})

        for Rtype in self.f_type:
            self.MakeOneUnit(pdf_page, {'Type': Rtype})

        pdf_page.close()




def one_graph(date, ti):
    currentdir = pathlib.Path(sys.argv[0]).parent/'graph'
    netfilename = str(date)+'_'+str(ti)+'graph.csv'
    netfile = currentdir/netfilename
    aa = Graph(netfile)
    aa.PngSave()


def main():
    one_graph('20220215', '103702') # Figure 3E

if __name__ == '__main__':
    main()
